/*
 * Copyright 2004-2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.shale.dialog;

import java.io.Serializable;

/**
 * <p>{@link Status} represents the current status of executing a
 * {@link Dialog}, plus parent {@link Dialog}s that have executing
 * subordinate {@link Dialog}s as part of their own processing.
 * Instances are stored in session scope, so the implementation class
 * MUST be <code>Serializable</code>, and MUST NOT maintain references
 * to application scope variabes.</p>
 *
 * <p>Associated with each {@link Position}, applications have the option
 * to store a reference to a generic "data" object which (because it is
 * referenced by a {@link Status} object stored in session scope under a
 * well known key), is conveniently accessible for binding and lookup
 * operations.  When a {@link Position} is popped, the corresponding
 * data object reference is also released, avoiding the need for the
 * application to manage "dialog scope" resources itself.</p>
 *
 * $Id: Status.java 419431 2006-07-06 04:38:39Z wsmoak $
 */

public interface Status extends Serializable {


    // -------------------------------------------------------------- Properties


    /**
     * <p>Return the data reference (if any) for the current {@link Position}.</p>
     */
    public Object getData();


    /**
     * <p>Set the data reference for the current {@link Position}.</p>
     *
     * @param data The new data reference
     */
    public void setData(Object data);


    /**
     * <p>Return the name of the currently executing dialog (if any);
     * otherwise, return <code>null</code>.</p>
     */
    public String getDialogName();


    /**
     * <p>Return the name of the currently executing dialog's state
     * (if any); otherwise, return <code>null</code>.</p>
     */
    public String getStateName();


    // ---------------------------------------------------------- Public Methods


    /**
     * <p>Return a {@link Position} representing the currently executing
     * dialog and state (if any); otherwise, return <code>null</code>.</p>
     */
    public Position peek();


    /**
     * <p>Pop the currently executing {@link Position} and return the
     * previously executing {@link Position} (if any); otherwise, return
     * <code>null</code>.</p>
     *
     * @exception IllegalStateException if there is no currently executing
     *  dialog
     */
    public Position pop() throws IllegalStateException;


    /**
     * <p>Push the specified {@link Position}, making it the currently
     * executing one.</p>
     *
     * @param position The new currently executing {@link Position}
     *
     * @exception IllegalArgumentException if <code>position</code>
     *  is <code>null</code>
     */
    public void push(Position position) throws IllegalArgumentException;


    /**
     * <p>Transition to the specified state in the currently executing
     * dialog.</p>
     *
     * @param stateName Name of the new state to be executed
     *
     * @exception IllegalStateException if there is no currently executing
     *  dialog
     */
    public void transition(String stateName) throws IllegalStateException;


    // --------------------------------------------------- Position Nested Class


    /**
     * <p>A <code>Position</code> represents a specific current {@link State}
     *  in a specific current {@link Dialog}.</p>
     */
    static class Position implements Serializable {

        /**
         * <p>Construct a new {@link Status.Position} for the specified
         * parameters.</p>
         *
         * @param dialogName Name of the executing dialog
         * @param stateName Name of the current state
         */
        public Position(String dialogName, String stateName) {
            this.dialogName = dialogName;
            this.stateName = stateName;
        }

        /**
         * <p>The dialog name of this position.</p>
         */
        private String dialogName = null;

        /**
         * <p>The state name of this position.</p>
         */
        private String stateName = null;

        /**
         * <p>Return the dialog name corresponding to this position.</p>
         */
        public String getDialogName() {
            return this.dialogName;
        }

        /**
         * <p>Return the state name corresponding to this position.</p>
         */
        public String getStateName() {
            return this.stateName;
        }

        /**
         * <p>Set the state name corresponding to this position.</p>
         *
         * @param stateName The new state name
         */
        public void setStateName(String stateName) {
            this.stateName = stateName;
        }


        /**
         * <p>Return the hash code for this {@link Status.Position}.</p>
         */
        public int hashCode() {
            return dialogName.hashCode() ^ stateName.hashCode();
        }


        /**
         * <p>Return <code>true</code> if the specified object is a
         * {@link Status.Position} with the same dialog and state names.
         *
         * @param o The object to check for equality
         */
        public boolean equals(Object o) {
            if (o instanceof Position) {
                Position p = (Position) o;
                return this.dialogName.equals(p.getDialogName())
                       && this.stateName.equals(p.getStateName());
            } else {
                return false;
            }
        }


        /**
         * <p>Return a string representation of this position.</p>
         */
        public String toString() {
            return "Position[dialogName=" + this.dialogName
                    + ",stateName=" + this.stateName + "]";
        }


    }


}
