/*
 * Copyright 2004-2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.shale.dialog.config;

import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.ServletContext;

import org.apache.commons.chain.Command;
import org.apache.commons.chain.Context;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.shale.dialog.Globals;


/**
 * <p>Command executed at web application startup to configure the
 * dialogs used by this application, storing an appropriate <code>Map</code>
 * in application scope.</p>
 *
 * $Id: ConfigurationInit.java 419431 2006-07-06 04:38:39Z wsmoak $
 */
public final class ConfigurationInit implements Command {

    /**
     * <p>The default resources file to load from (if it exists)
     * after any specified resources files are loaded.</p>
     */
    private static final String DEFAULT = "/WEB-INF/dialog-config.xml";


    /**
     * <p>Log instance for this class.</p>
     */
    private final Log log = LogFactory.getLog(getClass());


    /**
     * <p>Configure dialog information and store it in application scope.</p>
     *
     * @param context Execution context for this command
     *
     * @exception Exception if an executed command throws an exception
     */
    public boolean execute(Context context) throws Exception {

        // Acquire a reference to our ServletContext and initialize
        // the Map we will use to store configuration information.
        ServletContext scontext = (ServletContext) context.get("context");
        Map map = new HashMap();
        scontext.setAttribute(Globals.DIALOGS, map);

        // Initialize the parser utility we will use to parse resources
        ConfigurationParser parser = new ConfigurationParser();
        parser.setDialogs(map);
        parser.setValidating(true);

        // Acquire the list of context relative resource paths
        // for resources we will be loading
        String resources = scontext.getInitParameter(Globals.CONFIGURATION_PARAM);
        if (resources == null) {
            resources = "";
        }

        // Parse any resources specified by the developer
        String path = null;
        while (true) {
            int comma = resources.indexOf(",");
            if (comma < 0) {
                path = resources.trim();
                resources = "";
            } else {
                path = resources.substring(0, comma).trim();
                resources = resources.substring(comma + 1);
            }
            if (path.length() < 1) {
                break;
            }
            URL url = scontext.getResource(path);
            if (url == null) {
                throw new IllegalArgumentException(path);
            }
            parser.setResource(url);
            parser.parse();
        }

        // Parse the default resource (if it exists)
        URL url = scontext.getResource(DEFAULT);
        if (url != null) {
            parser.setResource(url);
            parser.parse();
        }

        // Allow other configuration activities to proceed
        if (log.isDebugEnabled()) {
            log.debug("Total dialogs configured: " + map.size());
        }
        return false;

    }


}
