/*
 * Copyright 2004-2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.shale.dialog.faces;

import java.util.Map;

import javax.faces.application.NavigationHandler;
import javax.faces.application.ViewHandler;
import javax.faces.component.UIViewRoot;
import javax.faces.context.FacesContext;
import javax.faces.el.MethodBinding;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.shale.dialog.ActionState;
import org.apache.shale.dialog.Dialog;
import org.apache.shale.dialog.EndState;
import org.apache.shale.dialog.Globals;
import org.apache.shale.dialog.State;
import org.apache.shale.dialog.Status;
import org.apache.shale.dialog.SubdialogState;
import org.apache.shale.dialog.Transition;
import org.apache.shale.dialog.ViewState;
import org.apache.shale.dialog.impl.StatusImpl;
import org.apache.shale.util.Messages;

/**
 * <p>An implementation of <code>NavigationHandler</code> that provides
 * support for dialog management according to the rules specified below.
 * It delegates handling to the standard implementation when there is no
 * current dialog, or when an unrecognized outcome is returned.</p>
 * <ul>
 * <li>When <code>handleNavigation()</code> is called with a logical outcome
 *     of the form <em>dialog:xxx</em>, and there is no current dialog
 *     in progress, the {@link Dialog} with name <code>xxx</code> is
 *     selected, and the specified starting state is selected.</li>
 * <li>If the current state is an {@link ActionState}, a
 *     <code>MethodBinding</code> is created, and the specified method
 *     is called.  The returned outcome is used to transition to
 *     another state within this dialog.</li>
 * <li>If the current state is a {@link SubdialogState}, the current
 *     position is pushed onto a stack, and the dialog specified by the
 *     {@link SubdialogState} is entered at its specified starting state.
 *     When the {@link EndState} for the subordinate dialog returns an
 *     outcome, the previous state is popped from the stack, and the
 *     outcome is used to transition to another state within this dialog.</li>
 * <li>If the current state is a {@link ViewState}, the specified view
 *     identifier will be used to create a new JavaServer Faces view, and
 *     then cause it to be rendered.  When the subsequent form submit is
 *     performed, and <code>handleNavigation()</code> is called again, the
 *     logical outcome (returned by whatever action method was invoked)
 *     will be used to select the transition to the next state in the
 *     current dialog.</li>
 * <li>If the current state is an {@link EndState}, processing proceeds
 *     depending upon whether or not there is a view identifier specified:
 *     <ul>
 *     <li>If there is a view identifier specified, it is created and
 *         rendered, as for a {@link ViewState}.  When the subseqeent
 *         form submit is performed, the parent dialog will be popped
 *         from the stack, and this outcome will be passed to it for
 *         transition processing (if there is no parent dialog, this
 *         outcome will be delegated to the standard navigation handler).</li>
 *     <li>If there is no view identifier specified, the outcome that
 *         triggered transition to this state will be passed to the
 *         parent dialog (if any) or delegated to the standard
 *         navigation handler.</li>
 *     </ul>
 * </ul>
 *
 * $Id: DialogNavigationHandler.java 419431 2006-07-06 04:38:39Z wsmoak $
 */

public final class DialogNavigationHandler extends NavigationHandler {


    // ------------------------------------------------------------ Constructors


    /**
     * <p>Create a new {@link DialogNavigationHandler}, wrapping the
     * specified standard navigation handler implementation.</p>
     *
     * @param handler Standard <code>NavigationHandler</code> we are wrapping
     */
    public DialogNavigationHandler(NavigationHandler handler) {

        this.handler = handler;

    }


    // -------------------------------------------------------- Static Variables


    /**
     * <p>An empty parameters list to pass to the <code>invoke()</code>
     * method of a <code>MethodBinding</code>.</p>
     */
    private static final Object[] PARAMETERS = new Object[0];


    /**
     * <p>The default prefix on a logical outcome String that indicates the remainder
     * of the string is the name of a {@link Dialog} to be entered.  The actual value
     * used can be set by defining a context initialization parameter for key
     * {@link Globals.PREFIX}.</p>
     */
    public static final String PREFIX = "dialog:";


    /**
     * <p>Parameter signature for the method bindings we create for
     * an {@link ActionState}.</p>
     */
    public static final Class[] SIGNATURE = new Class[0];


    /**
     * <p>Localized messages for this class.</p>
     */
    private static final Messages messages =
        new Messages("org.apache.shale.resources.Bundle");


    // ------------------------------------------------------ Instance Variables


    /**
     * <p>The standard <code>NavigationHandler</code> implementation that
     * we are wrapping.</p>
     */
    private NavigationHandler handler = null;


    /**
     * <p>The <code>Log</code> instance for this class.</p>
     */
    private final Log log = LogFactory.getLog(getClass());


    /**
     * <p>Session scope key under which we will store a {@link Status}
     * instance, if it is not present already.</p>
     */
    private String statusKey = null; // Cached on first use



    // ----------------------------------------------- NavigationHandler Methods


    /**
     * <p>Handle the navigation request implied by the specified parameters.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param fromAction The action binding expression that was evaluated
     *  to retrieve the specified outcome (if any)
     * @param outcome The logical outcome returned by the specified action
     *
     * @exception IllegalArgumentException if the configuration information
     *  for a previously saved position cannot be found
     * @exception IllegalArgumentException if an unknown State type is found
     */
    public void handleNavigation(FacesContext context, String fromAction,
                                 String outcome) {

        if (log.isDebugEnabled()) {
            log.debug("handleNavigation(viewId=" +
                      context.getViewRoot().getViewId() +
                      ",fromAction=" + fromAction +
                      ",outcome=" + outcome + ")");
        }

        // If we are not executing a dialog, delegate to the standard handler
        // unless the outcome starts with our prefix to start a dialog
        Status status = getStatus(context, false);
        String prefix = prefix(context);
        if (status == null) {
            if ((outcome != null) && outcome.startsWith(prefix)) {
                start(context, outcome.substring(prefix.length()));
            } else {
                handler.handleNavigation(context, fromAction, outcome);
            }
            return;
        }
        Status.Position position = status.peek();
        if (position == null) {
            if ((outcome != null) && outcome.startsWith(prefix)) {
                start(context, outcome.substring(prefix.length()));
            } else {
                handler.handleNavigation(context, fromAction, outcome);
            }
            return;
        }

        // Postprocess based on the nature of the current state
        Dialog dialog = getDialog(context, position.getDialogName());
        if (dialog == null) {
            throw new IllegalArgumentException(messages.getMessage("dialog.noDialog",
                                               new Object[] { position.getDialogName() }));
        }
        State state = dialog.findState(position.getStateName());
        if (state == null) {
            throw new IllegalArgumentException(messages.getMessage("dialog.noState",
                                               new Object[] { position.getStateName(),
                                                              position.getDialogName() }));
        }
        postprocess(context, status, state, outcome);

    }


    // --------------------------------------------------------- Private Methods


    /**
     * <p>Call the method specified by this {@link ActionState}, and
     * transition appropriately based on the outcome.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param status {@link Status} for the current user
     * @param state {@link ActionState} to be handled
     */
    private void action(FacesContext context, Status status, ActionState state) {

        assert context != null;
        assert status != null;
        assert state != null;

        if (log.isDebugEnabled()) {
            log.debug("action(state=" + state + ")");
        }

        // Execute the specified method and retrieve the logical outcome
        MethodBinding mb =
          context.getApplication().createMethodBinding(state.getMethod(),
                                                       SIGNATURE);
        String outcome = (String) mb.invoke(context, PARAMETERS);

        // Use the logical outcome to transition to the next state
        State nextState = transition(status, state, outcome);

        // Preprocess as necessary for the next state
        preprocess(context, status, nextState, outcome);

    }


    /**
     * <p>Return the {@link Dialog} configuration instance for the
     * specified dialog name, if any; otherwise, return <code>null</code>.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param dialogName Name of the requested dialog
     *
     * @exception IllegalStateException if dialog configuration metadata
     *  has not yet been processed
     */
    private Dialog getDialog(FacesContext context, String dialogName) {

        assert context != null;
        assert dialogName != null;

        Map map = (Map)
          context.getExternalContext().getApplicationMap().get(Globals.DIALOGS);
        if (map == null) {
            throw new IllegalStateException(messages.getMessage("dialog.unconfigured"));
        }
        return (Dialog) map.get(dialogName);

    }


    /**
     * <p>Return the {@link Status} instance for the current user,
     * optionally creating a new one if necessary.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param create Flag selecting whether to create if necessary
     */
    private Status getStatus(FacesContext context, boolean create) {

        assert context != null;

        Map map = context.getExternalContext().getSessionMap();
        String key = getStatusKey(context);
        assert key != null;
        Status status = (Status) map.get(key);
        if (create && (status == null)) {
            status = new StatusImpl();
            map.put(key, status);
        }
        return status;

    }


    /**
     * <p>Return the session scope attribute key under which we will
     * store a {@link Status} instance for the current user.  The value
     * is specified by a context init parameter named by constant
     * <code>Globals.STATUS_PARAM</code>, or defaults to the value
     * specified by constant <code>Globals.STATUS</code>.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     */
    private String getStatusKey(FacesContext context) {

        if (statusKey == null) {
            statusKey =
              context.getExternalContext().getInitParameter(Globals.STATUS_PARAM);
            if (statusKey == null) {
                statusKey = Globals.STATUS;
            }
        }
        return statusKey;

    }


    /**
     * <p>Postprocess the specified state, which we retrieved from
     * the current {@link Status} for this user.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param status Current {@link Status} for this user
     * @param state The {@link State} we retrieved based on the {@link Status}
     * @param outcome Original outcome from handler
     *
     * @exception IllegalArgumentException if the current state is not
     *  a ViewState
     */
    private void postprocess(FacesContext context, Status status,
                             State state, String outcome) {

        if (log.isDebugEnabled()) {
            log.debug("preprocess(state=" + state + ",outcome=" + outcome + ")");
        }

        // The only type of State with postprocessing is a ViewState
        if (state instanceof ViewState) {
            State nextState = transition(status, state, outcome);
            preprocess(context, status, nextState, outcome);
        } else {
            throw new IllegalArgumentException(messages.getMessage("dialog.noPostProcess",
                                               new Object[] { state.getName(),
                                                              state.getClass().getName() }));
        }

    }


    /**
     * <p>Return the prefix value to use for determining whether the specified
     * logical outcome represents a request to initiate a named dialog.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     */
    private String prefix(FacesContext context) {

        String prefix = context.getExternalContext().getInitParameter(Globals.PREFIX);
        if (prefix == null) {
            prefix = this.PREFIX;
        }
        return prefix;

    }


    /**
     * <p>Preprocess a transition to the specified state that has just been
     * completed.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param status Current {@link Status} for this user
     * @param state The {@link State} we just transitioned to
     * @param outcome The outcome that caused us to transition to this
     *  {@link State}
     *
     * @exception IllegalArgumentException if the specified state is not of
     *  a known type
     */
    private void preprocess(FacesContext context, Status status,
                            State state, String outcome) {

        assert context != null;
        assert status != null;
        assert state != null;

        if (log.isDebugEnabled()) {
            log.debug("preprocess(state=" + state + ",outcome=" + outcome + ")");
        }

        if (state instanceof ActionState) {
            // Recursively call to process a chained ActionState
            action(context, status, (ActionState) state);
        } else if (state instanceof EndState) {
            String viewId = ((EndState) state).getViewId();
            if (viewId != null) {
                render(context, viewId);
            }
            Status.Position position = status.pop();
            if (position != null) {
                Dialog dialog = getDialog(context, position.getDialogName());
                if (dialog == null) {
                    throw new IllegalArgumentException(position.getDialogName());
                }
                state = dialog.findState(position.getStateName());
                if (state == null) {
                    throw new IllegalArgumentException(dialog.getStart());
                }
                if (log.isDebugEnabled()) {
                    log.debug("-->resume(state=" + state + ",outcome=" + outcome + ")");
                }
                State nextState = transition(status, state, outcome);
                preprocess(context, status, nextState, outcome);
            }
        } else if (state instanceof SubdialogState) {
            start(context, ((SubdialogState) state).getDialogName());
        } else if (state instanceof ViewState) {
            render(context, ((ViewState) state).getViewId());
        } else {
            throw new IllegalArgumentException(messages.getMessage("dialog.noPreProcess",
                                               new Object[] { state.getName(),
                                                              state.getClass().getName() }));
        }

    }


    /**
     * <p>Render the view corresponding to the specified view identifier.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param viewId View identifier to be rendered, or <code>null</code>
     *  to rerender the current view
     */
    private void render(FacesContext context, String viewId) {

        assert context != null;

        if (log.isDebugEnabled()) {
            log.debug("render(viewId=" + viewId + ")");
        }

        // Stay on the same view if requested
        if (viewId == null) {
            return;
        }

        // Create the specified view so that it can be rendered
        ViewHandler vh = context.getApplication().getViewHandler();
        UIViewRoot view = vh.createView(context, viewId);
        view.setViewId(viewId);
        context.setViewRoot(view);

    }


    /**
     * <p>Select the specified {@link Dialog}, and transition to the start
     * {@link State}.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param dialogName Name of the dialog to be started
     */
    private void start(FacesContext context, String dialogName) {

        if (log.isDebugEnabled()) {
            log.debug("start(dialogName=" + dialogName + ")");
        }

        // Acquire the State configuration for the starting state
        Dialog dialog = getDialog(context, dialogName);
        if (dialog == null) {
            throw new IllegalArgumentException(dialogName);
        }
        State state = dialog.findState(dialog.getStart());
        if (state == null) {
            throw new IllegalArgumentException(dialog.getStart());
        }

        // Register this user as being in the specfied State
        // and perform any required preprocessing
        Status status = getStatus(context, true);
        assert status != null;
        status.push(new Status.Position(dialog.getName(), state.getName()));
        preprocess(context, status, state, dialogName);

    }


    /**
     * <p>Transition from the specified {@link State} to a new {@link State}
     * based on the specified logical outcome, and return the new {@link State}.
     * When trying to identify an appropriate {@link Transition}, first check
     * for a {@link Transition} directly owned by the origin {@link State};
     * if there is none, check for a global {@link Transition} associated with
     * the {@link Dialog} that owns the origin {@link State}.</p>
     *
     * <p><strong>NOTE</strong> For consistency with standard JavaServer Faces
     * navigation, transition from a {@link ViewState} with a <code>null</code>
     * outcome stays in the same {@link ViewState}.</p>
     *
     * @param status Current {@link Status} for this user
     * @param state Current {@link State} for this user
     * @param outcome Logical outcome used to select next {@link State}
     *
     * @exception IllegalArgumentException if no next {@link State}
     *  can be identified
     */
    private State transition(Status status, State state, String outcome) {

        if (log.isDebugEnabled()) {
            log.debug("transition(state=" + state + ",outcome=" + outcome + ")");
        }

        // Transition from a view state with a null transition means stay where we are
        if ((state instanceof ViewState) && (outcome == null)) {
            if (log.isDebugEnabled()) {
                log.debug("  --> Stay in current state");
            }
            return state;
        }

        // Identify the appropriate Transition
        Transition transition = state.findTransition(outcome);
        if (transition == null) {
            transition = state.getDialog().findTransition(outcome);
        }
        if (transition == null) {
            throw new IllegalArgumentException(messages.getMessage("dialog.noTransition",
                                               new Object[] { outcome,
                                                              state.getName(),
                                                              state.getDialog().getName() }));
        }

        // Select the next State based on this Transition
        State nextState = state.getDialog().findState(transition.getTarget());
        if (nextState == null) {
            throw new IllegalArgumentException(messages.getMessage("dialog.noNextState",
                                               new Object[] { outcome,
                                                              state.getName(),
                                                              state.getDialog().getName(),
                                                              transition.getTarget() }));
        }

        // Transition to the next State and return it
        if (log.isDebugEnabled()) {
            log.debug("-->nextState=" + nextState);
        }
        status.transition(nextState.getName());
        return nextState;

    }


}
