/*
 * Copyright 2004-2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.shale.dialog.impl;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.shale.dialog.Dialog;
import org.apache.shale.dialog.Status;

/**
 * <p>Class used to record the current status of executing a {@link Dialog},
 * plus parent {@link Dialog}s that have executed subordinate {@link Dialog}s
 * as part of their own processing.  Instances are stored in session
 * scope, so this class <strong>MUST</strong> be serializable, and
 * <strong>MUST NOT</strong> maintain references to application scope
 * variabes.</p>
 *
 * <p>Associated with each {@link org.apache.shale.dialog.Status.Position},
 * applications have the option to store a reference to a generic
 * <code>data</code> object which (because it is referenced by a
 * {@link Status} object stored in session scope under a well known key),
 * is conveniently accessible for binding and lookup operations.  When a
 * {@link org.apache.shale.dialog.Status.Position} is popped, the corresponding
 * data object reference is also released, avoiding the need for the
 * application to manage "dialog scope" resources itself.</p>
 *
 * $Id: StatusImpl.java 419431 2006-07-06 04:38:39Z wsmoak $
 */

public class StatusImpl implements Status {


    // ------------------------------------------------------ Instance Variables


    /**
     * <p>Log instance for this instance.</p>
     */
    private final Log log = LogFactory.getLog(this.getClass());


    /**
     * <p>The stack of currently stored
     * {@link org.apache.shale.dialog.Status.Position}s.  If there are no
     * entries, there is no currently executing dialog.</p>
     */
    private List positions = new ArrayList();


    /**
     * <p>References to data objects that are associated with the
     * {@link org.apache.shale.dialog.Status.Position}s at corresponding
     * indexes of the <code>positions</code> list.</p>
     */
    private List references = new ArrayList();


    // -------------------------------------------------------------- Properties


    /**
     * {@inheritDoc}
     */
    public Object getData() {

        // NOTE - all locks on "positions" for consistency
        synchronized (positions) {
            int index = positions.size() - 1;
            if (index < 0) {
                return null;
            }
            return references.get(index);
        }

    }


    /**
     * {@inheritDoc}
     */
    public void setData(Object data) {

        // NOTE - all locks on "positions" for consistency
        synchronized (positions) {
            int index = positions.size() - 1;
            if (index < 0) {
                throw new IllegalStateException();
            }
            references.set(index, data);
        }
    }


    /**
     * {@inheritDoc}
     */
    public String getDialogName() {

        Position position = peek();
        if (position != null) {
            return position.getDialogName();
        } else {
            return null;
        }

    }


    /**
     * {@inheritDoc}
     */
    public String getStateName() {

        Position position = peek();
        if (position != null) {
            return position.getStateName();
        } else {
            return null;
        }

    }


    // ---------------------------------------------------------- Public Methods


    /**
     * {@inheritDoc}
     */
    public Position peek() {

        synchronized (positions) {
            int index = positions.size() - 1;
            if (log.isTraceEnabled()) {
                log.trace("peek() --> " +
                          ((index < 0) ? null : (Position) positions.get(index)) +
                          "@" + index);
            }
            if (index < 0) {
                return null;
            }
            return (Position) positions.get(index);
        }

    }


    /**
     * {@inheritDoc}
     */
    public Position pop() {

        synchronized (positions) {
            int index = positions.size() - 1;
            if (index < 0) {
                throw new IllegalStateException();
            }
            positions.remove(index);
            references.remove(index);
            if (log.isDebugEnabled()) {
                log.debug("pop(" + index + ") --> " +
                          ((index > 0) ? (Position) positions.get(index - 1) : null));
            }
            if (index > 0) {
                return (Position) positions.get(index - 1);
            } else {
                return null;
            }
        }

    }


    /**
     * {@inheritDoc}
     */
    public void push(Position position) {

        if (position == null) {
            throw new IllegalArgumentException();
        }
        synchronized (positions) {
            positions.add(position);
            references.add(null);
        }

        if (log.isDebugEnabled()) {
            log.debug("push(" + position + ")@" + (positions.size() - 1));
        }

    }


    /**
     * {@inheritDoc}
     */
    public void transition(String stateName) {

        synchronized (positions) {
            int index = positions.size() - 1;
            if (index < 0) {
                throw new IllegalArgumentException();
            }
            ((Position) positions.get(index)).setStateName(stateName);
        }

    }


}
