/*
 * Copyright 2004-2005 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.shale.remote;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.faces.model.SelectItem;

import org.apache.commons.chain.Command;
import org.apache.commons.chain.Context;

/**
 * <p>Convenience base class for remote commands that retrieve arrays
 * of <code>SelectItem</code> beans, optionally filtered by matching the
 * label against a test String, and rendering the result as an XML document.</p>
 *
 * @deprecated Use corresponding functionality from the
 *  <code>org.apache.shale.remoting</code> package
 *
 * $Id: AbstractSelectItems.java 419431 2006-07-06 04:38:39Z wsmoak $
 */

public abstract class AbstractSelectItems implements Command {
    
    
    // -------------------------------------------------------- Static Variables


    /**
     * <p>Default request parameter name used by the <code>test()</code>
     * method to extract the comparison string.</p>
     */
    protected static final String PARAMETER = "prefix";
    

    // ------------------------------------------------------ Instance Variables


    // -------------------------------------------------------------- Properties


    /**
     * <p>Return the name of the inner element for the "description" property
     * with which our response is created.  The default implementation
     * returns <code>description</code>.</p>
     */
    protected String getInnerDescriptionElement() {
        return "description";
    }


    /**
     * <p>Return the name of the inner element for the "label" property
     * with which our response is created.  The default implementation
     * returns <code>label</code>.</p>
     */
    protected String getInnerLabelElement() {
        return "label";
    }


    /**
     * <p>Return the name of the inner element for the "value" property
     * with which our response is created.  The default implementation
     * returns <code>value</code>.</p>
     */
    protected String getInnerValueElement() {
        return "value";
    }


    /**
     * <p>Return the name of the inner element for the overall item
     * with which our response is created.  The default implementation
     * returns <code>item</code>.</p>
     */
    protected String getInnerElement() {
        return "item";
    }


    /**
     * <p>Return the name of the outer element with which our response is
     * created.  The default implementation returns <code>items</code>.</p>
     */
    protected String getOuterElement() {
        return "items";
    }


    // --------------------------------------------------------- Command Methods


    /**
     * <p>Perform the appropriate matches, and return an XML document containing them.
     * The default implementation uses the following protected methods:</p>
     * <ul>
     * <li><code>test()</code> to retrieve the test value for this request.</li>
     * <li><code>legal()</code> to retrieve the legal items to be compared
     *     against the test value.</li>
     * <li><code>match()</code> to return the legal items that match the
     *     test value.</li>
     * <li><code>render()</code> to render the result as an XML document.</li>
     * </ul>
     *
     * @param context {@link RemoteContext} for this request
     */
    public boolean execute(Context context) throws Exception {

        RemoteContext rcontext = (RemoteContext) context;
        String test = test(rcontext);
        SelectItem legal[] = legal(rcontext, test);
        SelectItem matches[] = match(rcontext, test, legal);
        render(rcontext, matches);
        return true; // Response is now complete

    }


    // ------------------------------------------------------ Protected Methods


    /**
     * <p>Return the (possibly empty) set of legal items that are currently
     * available, based on the specified test string.  This method is used
     * by the default implementation of <code>match()</code>, but may be
     * made unnecessary by a custom implementation of that method.</p>
     *
     * @param context {@link RemoteContext} for this request
     * @param test Test value optionally used to select legal values
     */
    protected abstract SelectItem[] legal(RemoteContext context, String test);


    /**
     * <p>Return the (possibly empty) set of matching items based on
     * filtering the specified test string against the specified set of legal
     * items.  By default, a case insensitive comparison of the test String
     * to the beginning of each item's <code>label</code> is performed.
     * If no matches are found, a zero length array is returned.  If the
     * test string is null or zero length, all legal items are considered
     * to be matches, and are returned.</p>
     *
     * @param context {@link RemoteContext} for this request
     * @param test Test string to compare to the possible legal values
     * @param legal Legal values that might be matched by the specified prefix
     */
    protected SelectItem[] match(RemoteContext context, String test, SelectItem legal[]) {

        if ((test == null) || (test.length() == 0)) {
            return legal;
        }
        if (legal == null) {
            return new SelectItem[0];
        }
        List list = new ArrayList();
        String match = test.toLowerCase();
        for (int i = 0; i < legal.length; i++) {
            if (legal[i].getLabel().toLowerCase().startsWith(match)) {
                list.add(legal[i]);
            }
        }
        return (SelectItem[]) list.toArray(new SelectItem[list.size()]);

    }


    /**
     * <p>Render an XML document response containing the specified matching
     * values, representing each as an element whose name is returned by the
     * <code>getInnerElement()</code> method.  The overall response is contained
     * in an element whose name is returned by the <code>getOuterElement()</code>
     * method.</p>
     *
     * @param context <code>RemoteContext</code> for the current request
     * @param matches String values to be rendered
     *
     * @exception IOException if an input/output error occurs while rendering
     */
    protected void render(RemoteContext context, SelectItem matches[]) throws IOException {

        // Set up the response we will create
        context.setResponseContentType("text/xml");
        context.setResponseEncoding("UTF-8");
        ResponseWrapper wrapper =
          new ResponseWrapper(context.getResponseWriter(), "UTF-8");
        String inner = getInnerElement();
        String innerDescription = getInnerDescriptionElement();
        String innerLabel = getInnerLabelElement();
        String innerValue = getInnerValueElement();
        String outer = getOuterElement();

        // Generate the response content
        wrapper.startDocument();
        wrapper.startElement(outer);
        wrapper.writeNewline();
        for (int i = 0; i < matches.length; i++) {
            if (matches[i] == null) {
                continue;
            }
            wrapper.startElement(inner);
            wrapper.writeNewline();
            Object value = matches[i].getValue();
            if (value != null) {
                wrapper.startElement(innerValue);
                wrapper.writeText(value.toString());
                wrapper.endElement(innerValue);
                wrapper.writeNewline();
            }
            String label = matches[i].getLabel();
            if (label != null) {
                wrapper.startElement(innerLabel);
                wrapper.writeText(label);
                wrapper.endElement(innerLabel);
                wrapper.writeNewline();
            }
            String description = matches[i].getDescription();
            if (description != null) {
                wrapper.startElement(innerDescription);
                wrapper.writeText(description);
                wrapper.endElement(innerDescription);
                wrapper.writeNewline();
            }
            wrapper.endElement(inner);
            wrapper.writeNewline();
        }
        wrapper.endElement(outer);
        wrapper.endDocument();

    }


    /**
     * <p>Return the test value from the current context.  The default
     * implementation uses a request parameter named by symbolic constant
     * PARAMETER.  The returned value may be <code>null</code> if no such
     * request parameter was included on this request.</p>
     *
     * @param context {@link RemoteContext} for the current request
     */
    protected String test(RemoteContext context) {

        String prefixes[] =
          (String[]) context.getRequestParameters().get(PARAMETER);
        if (prefixes == null) {
            prefixes = new String[0];
        }
        return (prefixes.length > 0) ? prefixes[0] : null;

    }

}
