/*
 * Copyright 2004-2005 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.shale.remote;

import java.io.IOException;
import java.io.StreamTokenizer;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

import javax.faces.FactoryFinder;
import javax.faces.context.FacesContext;
import javax.faces.context.FacesContextFactory;
import javax.faces.lifecycle.Lifecycle;
import javax.faces.lifecycle.LifecycleFactory;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.chain.Catalog;
import org.apache.commons.chain.CatalogFactory;
import org.apache.commons.chain.Command;
import org.apache.commons.chain.Context;
import org.apache.shale.faces.ShaleWebContext;

/**
 * <p>Implementation of <code>Command</code> (from the Commons Chain API)
 * that executes an appropriate <code>Command</code> or <code>Chain</code>
 * to implement remote requests from client side components.</p>
 *
 * @deprecated Use corresponding functionality from the
 *  <code>org.apache.shale.remoting</code> package
 *
 * $Id: RemoteCommand.java 419431 2006-07-06 04:38:39Z wsmoak $
 */
public class RemoteCommand implements Command {
    

    // -------------------------------------------------------------- Properties


    /**
     * <p>The name of the <code>Catalog</code> in which commands are searched
     * for to execute specific requests.  The default value is "remote".</p>
     */
    private String catalogName = "remote";
    public String getCatalogName() { return this.catalogName; }
    public void setCatalogName(String catalogName)
    { this.catalogName = catalogName; }


    /**
     * <p>The JavaServer Faces lifecycle identifier of the <code>Lifecycle</code>
     * instance we will be using.</p>
     */
    private String lifecycleId = LifecycleFactory.DEFAULT_LIFECYCLE;
    public String getLifecycleId() { return this.lifecycleId; }
    public void setLifecycleId(String lifecycleId) { this.lifecycleId = lifecycleId; }


    /**
     * <p>Comma-delimited list of regular expression pattern(s) the context
     * relative URI of this request must match in order to be processed.  The
     * default value matches any request that ends with ".remote".</p>
     */
    private String patterns = "\\S*\\.remote";
    public String getPatterns() { return this.patterns; }
    public void setPatterns(String patterns) { this.patterns = patterns; }


    /**
     * <p>Flag indicating whether or not we should construct a
     * JavaServer Faces <code>FacesContext</code> for each request.
     * Default value is <code>true</code>.</p>
     */
    private boolean usingFacesContext = true;
    public boolean isUsingFacesContext() { return this.usingFacesContext; }
    public void setUsingFacesContext(boolean usingFacesContext) {
        this.usingFacesContext = usingFacesContext;
    }


    // --------------------------------------------------------- Command Methods


    /**
     * <p>If the context-relative URI for this request matches one of the
     * specified regular expression patterns, use the URI to look up the name
     * of a command in the specified catalog, construct an appropriate
     * {@link RemoteContext} for it, execute this command, and return
     * <code>true</code> to indicate that processing for this request has
     * been completed.  If the context-relative URI does not match one of
     * the specified regular expression patterns, return <code>false</code>
     * so that the request will be processed by the remainder of the standard
     * processing chain.</p>
     *
     * @param context <code>Context</code> for this request
     *
     * @exception Exception if thrown by a subordinate command
     */
    public boolean execute(Context context) throws Exception {

        // Calculate the context-relative URI for this request
        // FIXME - support portlet environments also
        ShaleWebContext swcontext = (ShaleWebContext) context;
        HttpServletRequest request = swcontext.getRequest();
        String servletPath = request.getServletPath();
        if (servletPath == null) {
            servletPath = "";
        }
        String pathInfo = request.getPathInfo();
        if (pathInfo == null) {
            pathInfo = "";
        }
        String uri = servletPath + pathInfo;

        // If this URI does not match our pattern(s), return false
        // to let the rest of the standard processing chain deal with it
        if (!match(uri)) {
            return false;
        }

        // Look up the appropriate command for this request
        Command command = catalog().getCommand(uri);
        if (command == null) {
            // FIXME - report an error somehow
            return true;
        }

        // If requested, create a FacesContext for this request.  This allows 
        // server side handler code to evaluate value binding and method 
        // binding expressions programatically outside of the JSF lifecycle.
        FacesContext facesContext = null;
        if (isUsingFacesContext()) {
            facesContext = facesContextFactory().getFacesContext(swcontext.getContext(),
                                                                 swcontext.getRequest(),
                                                                 swcontext.getResponse(),
                                                                 lifecycle());
        }

        // Construct a RemoteContext and execute this command with it
        // FIXME - do we want to carry over attributes from swcontext too?
        RemoteContext rcontext =
          new ServletRemoteContext(swcontext.getContext(),
                                   swcontext.getRequest(),
                                   swcontext.getResponse());
        command.execute(rcontext);

        // If we created a FacesContext for this request, clean it up appropriately
        if (facesContext != null) {
            facesContext.responseComplete(); // Not sure this is needed since we
                                             // did not actually run the lifecycle
        }

        // Return true to indicate this chain has been completed
        return true;

    }


    // --------------------------------------------------------- Private Methods


    /**
     * <p>The cached <code>Catalog</code> instance in which we will look
     * up commands to be executed.</p>
     */
    private Catalog catalog = null;


    /**
     * <p>Return the <code>Catalog</code> to be used for looking up commands,
     * caching this value for future use.</p>
     */
    private Catalog catalog() {

        if (catalog == null) {
            catalog = CatalogFactory.getInstance().getCatalog(catalogName);
        }
        return catalog;

    }


    /**
     * <p>The cached <code>FacesContextFactory</code> for this application.</p>
     */
    private FacesContextFactory facesContextFactory = null;


    /**
     * <p>Return the <code>FacesContextFactory</code> for this application,
     * caching it upon first retrieval.</p>
     */
    private FacesContextFactory facesContextFactory() {

        if (facesContextFactory == null) {
            facesContextFactory = (FacesContextFactory)
              FactoryFinder.getFactory(FactoryFinder.FACES_CONTEXT_FACTORY);
        }
        return facesContextFactory;

    }


    /**
     * <p>The cached <code>Lifecycle</code> for this application.</p>
     */
    private Lifecycle lifecycle = null;


    /**
     * <p>Return the <code>Lifecycle</code> for this application,
     * caching it upon first retrieval.</p>
     */
    private Lifecycle lifecycle() {

        if (lifecycle == null) {
            LifecycleFactory lifecycleFactory = (LifecycleFactory)
              FactoryFinder.getFactory(FactoryFinder.LIFECYCLE_FACTORY);
            lifecycle = lifecycleFactory.getLifecycle(getLifecycleId());
        }
        return lifecycle;

    }


    /**
     * <p>The cached regular expression pattern(s) we will use for matching.</p>
     */
    private Pattern match[] = null;


    /**
     * <p>Return <code>true</code> if the specified URI matches one of the
     * specified regular expressions.</p>
     *
     * @param uri Context-relative URI to be matched
     */
    private boolean match(String uri) {

        // Precreate our matching patterns if needed
        if (match == null) {
            match = precompile(patterns);
        }

        // Return true if the specified URI matches a pattern
        for (int i = 0; i < match.length; i++) {
            if (match[i].matcher(uri).matches()) {
                return true;
            }
        }
        return false;

    }


    /**
     * <p>Parse the specified string of comma-delimited (and optionally quoted,
     * if an embedded comma is required) regular expressions into an array
     * of precompiled <code>Pattern</code> instances that represent these
     * expressons.</p>
     *
     * @param expr Comma-delimited regular expressions
     */
    // FIXME refactor so this code is not duplicated from AbstractRegExpFilter
    private Pattern[] precompile(String expr) {

        if (expr == null) {
            return new Pattern[0];
        }

        // Set up to parse the specified expression
        StreamTokenizer st =
          new StreamTokenizer(new StringReader(expr));
        st.eolIsSignificant(false);
        st.lowerCaseMode(false);
        st.slashSlashComments(false);
        st.slashStarComments(false);
        st.wordChars(0x00, 0xff);
        st.quoteChar('\'');
        st.quoteChar('"');
        st.whitespaceChars(0, ' ');
        st.whitespaceChars(',', ',');
        List list = new ArrayList();
        int type = 0;

        // Parse each included expression
        while (true) {
            try {
                type = st.nextToken();
            } catch (IOException e) {
                // Can not happen
            }
            if (type == StreamTokenizer.TT_EOF) {
                break;
            } else if (type == StreamTokenizer.TT_NUMBER) {
                list.add(Pattern.compile("" + st.nval));
            } else if (type == StreamTokenizer.TT_WORD) {
                list.add(Pattern.compile(st.sval.trim()));
            } else {
                throw new IllegalArgumentException(expr);
            }
        }

        // Return the precompiled patterns as an array
        return (Pattern[]) list.toArray(new Pattern[list.size()]);

    }


}
