/*
 * Copyright 2004-2005 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.shale.remote;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Reader;
import java.io.Writer;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.chain.Context;

/**
 * <p>{@link RemoteContext} is an extension of <code>Context</code>
 * (from the Commons Chain package APIs) that encapsulates the remote
 * communications requirements of typical remoting applications.  An
 * instance of this class will be constructed by the framework to wrap
 * each request, facilitating the use of Commons Chain <code>Command</code>
 * or <code>Chain</code> instances to perform the requested remote
 * processing.</p>
 *
 * @deprecated Use corresponding functionality from the
 *  <code>org.apache.shale.remoting</code> package
 *
 * $Id: RemoteContext.java 419431 2006-07-06 04:38:39Z wsmoak $
 */

public interface RemoteContext extends Context  {
    
    
    // -------------------------------------------------------------- Properties


    /**
     * <p>Return a <code>Map</code> of the application-scope attributes
     * associated with this request.</p>
     */
    public Map getContextAttributes();


    /**
     * <p>Return a <code>Map</code> of the request-scope attributes
     * associated with this request.</p>
     */
    public Map getRequestAttributes();


    /**
     * <p>Return the content length specified by this request.</p>
     */
    public int getRequestContentLength();


    /**
     * <p>Return the content type specified by this request.</p>
     */
    public String getRequestContentType();


    /**
     * <p>Return the character encoding specified by this request (if any).
     * Otherwise, return <code>null</code>.</p>
     */
    public String getRequestEncoding();


    /**
     * <p>Return the <code>Locale</code> to use for this request.</p>
     */
    public Locale getRequestLocale();


    /**
     * <p>Return the HTTP method used to submit this request.</p>
     */
    public String getRequestMethod();


    /**
     * <p>Return an immutable <code>Map</code> containing the parameters
     * submitted with this request, keyed by parameter name.  Values in
     * this map will be String arrays of the value(s) for the corresponding
     * parameter name.</p>
     */
    public Map getRequestParameters();


    /**
     * <p>Return a <code>Reader</code> that may be used to consume the
     * character content of this request.</p>
     *
     * @exception IllegalStateException if <code>getRequestStream()</code>
     *  has already been called
     * @exception IOException if an intput/output error occurs
     */
    public Reader getRequestReader() throws IOException;

    
    /**
     * <p>Return an <code>InputStream</code> that may be used to consume the
     * binary content of this request.</p>
     *
     * @exception IllegalStateException if <code>getRequestReader()</code>
     *  has already been called
     * @exception IOException if an intput/output error occurs
     */
    public InputStream getRequestStream() throws IOException;


    /**
     * <p>Return the content length to be specified for this response.</p>
     */
    public int getResponseContentLength();


    /**
     * <p>Set the content length to be specified for this response.</p>
     *
     * @param contentLength The new content length
     */
    public void setResponseContentLength(int contentLength);


    /**
     * <p>Return the content type to be specified for this response.</p>
     */
    public String getResponseContentType();


    /**
     * <p>Set the content type to be specified for this response.</p>
     *
     * @param contentType The new content type
     */
    public void setResponseContentType(String contentType);


    /**
     * <p>Return the character encoding to be specified for this response.</p>
     */
    public String getResponseEncoding();


    /**
     * <p>Set the character encoding to be specified for this response.
     * To be effective, this method <code>MUST</code> be called before
     * <code>getResponseWriter()</code> is called.</p>
     *
     * @param encoding The new character encoding
     */
    public void setResponseEncoding(String encoding);


    /**
     * <p>Return an <code>OutputStream</code> that may be used to construct
     * the binary content of this response.</p>
     *
     * @exception IllegalStateException if <code>getResponseWriter()</code>
     *  has already been called
     * @exception IOException if an intput/output error occurs
     */
    public OutputStream getResponseStream() throws IOException;


    /**
     * <p>Return a <code>Writer</code> that may be used to construct
     * the character content of this response.</p>
     *
     * @exception IllegalStateException if <code>getResponseStream()</code>
     *  has already been called
     * @exception IOException if an intput/output error occurs
     */
    public Writer getResponseWriter() throws IOException;


    /**
     * <p>Return a <code>Map</code> of the session-scope attributes
     * associated with this request.  Calling this method will cause a
     * session to be created, if there is not currently one present.</p>
     */
    public Map getSessionAttributes();


}
