/*
 * Copyright 2004-2005 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.shale.remote;

import java.io.IOException;
import java.io.Writer;

/**
 * <p>Convenience wrapper around a <code>Writer</code> that includes utility
 * methods useful in constructing XML output.  Appropriate filtering and
 * wrapping for sensitive characters is performed automatically.</p>
 *
 * <p><strong>IMPLEMENTATION NOTE</strong> - This class makes many calls to
 * the <code>writer.write()</code> method.  Therefore, it is strongly suggested
 * that you provide a <code>Writer</code> instance that does buffering.</p>
 *
 * @deprecated Use corresponding functionality from the
 *  <code>org.apache.shale.remoting</code> package
 *
 * $Id: ResponseWrapper.java 419431 2006-07-06 04:38:39Z wsmoak $
 */
public class ResponseWrapper {
    

    // ------------------------------------------------------------ Constructors


    /**
     * <p>Construct a new {@link ResponseWrapper} wrapping the specified
     * <code>Writer</code>.  The character encoding will be assumed to be
     * <code>ISO-8859-1</code>.</p>
     *
     * @param writer <code>Writer</code> to which our output will be written
     */
    public ResponseWrapper(Writer writer) {

        this(writer, "ISO-8859-1");

    }


    /**
     * <p>Construct a new {@link ResponseWrapper} wrapping the specified
     * <code>Writer</code> that has been configured with the specified
     * character encoding.</p>
     *  
     * @param writer <code>Writer</code> to which our output will be written
     * @param encoding Character encoding used on this <code>Writer</code>
     */
    public ResponseWrapper(Writer writer, String encoding) {

        this.writer = writer;
        this.encoding = encoding;

    }


    // ------------------------------------------------------ Instance Variables


    /**
     * <p>The character encoding that has been set on the underlying
     * <code>Writer</code>.</p>
     */
    private String encoding = null;


    /**
     * <p>Flag indicating that we have a currently open element.</p>
     */
    private boolean open = false;


    /**
     * <p>The <code>Writer</code> to which our output will be written
     */
    private Writer writer = null;


    // ---------------------------------------------------------- Public Methods


    /**
     * <p>Create any appropriate prologue for the beginning of this document.</p>
     *
     * @exception IOException if an input/output error occurs
     */
    public void startDocument() throws IOException {

        writer.write("<?xml version=\"1.0\" encoding=\"" + encoding + "\"?>\n");

    }


    /**
     * <p>Write the start of an element, up to and including the element name.
     * Once this method has been called, clients can call the
     * <code>writeAttrbute()</code> or <code>writeURIAttribute()</code>
     * methods to add attributes and corresponding values.  The starting
     * element will be closed (that is, the trailing '>' character added)
     * on any subsequent call to <code>startElement()</code>,
     * <code>writeComment()</code>, <code>writeText()</code>,
     * <code>writeNewline()</code>,
     * <code>endElement()</code>, or <code>endDocument()</code>.</p>
     *
     * @param name Element name to be started
     *
     * @exception IOException if an input/output error occurs
     */
    public void startElement(String name) throws IOException {

        close();
        writer.write('<');
        writer.write(name);
        open = true;

    }


    /**
     * <p>Write an attribute name and corresponding value, after converting
     * that text to a String (if necessary), and after performing any
     * escaping appropriate for XML.  This method may only be called
     * after a call to <code>startElement()</code>,
     * and before the opened element has been closed.</p>
     *
     * @param name Attribute name to be written
     * @param value Attribute value to be written
     *
     * @exception IllegalStateException if no element is currently open
     * @exception IOException if an input/output error occurs
     */
    public void writeAttribute(String name, Object value) throws IOException {

        if (!open) {
            throw new IllegalStateException();
        }
        writer.write(" ");
        writer.write(name);
        writer.write("=\"");
        if (value instanceof String) {
            string(writer, (String) value);
        } else {
            string(writer, value.toString());
        }
        writer.write("\"");

    }


    /**
     * <p>Write a URI attribute name and corresponding value, after converting
     * that text to a String (if necessary), and after performing any
     * escaping appropriate for XML.  This method may only be called
     * after a call to <code>startElement()</code>,
     * and before the opened element has been closed.</p>
     *
     * <p><strong>IMPLEMENTATION NOTE</strong> - Currently, this just calls
     * through to <code>writeAttribute()</code>, so it serves as a placeholder
     * until it is determined whether URIs will need to be treated differently
     * than other attribute types.</p>
     *
     * @param name Attribute name to be written
     * @param value Attribute value to be written
     *
     * @exception IllegalStateException if no element is currently open
     * @exception IOException if an input/output error occurs
     */
    public void writeURIAttribute(String name, Object value) throws IOException {

        writeAttribute(name, value);

    }



    /**
     * <p>Write a comment containing the specified text, after converting
     * that text to a String (if necessary).  If there is an open element
     * that has been created by a call to <code>startElement()</code>, that
     * element will be closed first.</p>
     *
     * @param comment Commented text to be written
     *
     * @exception IOException if an input/output error occurs
     */
    public void writeComment(Object comment) throws IOException {

        close();
        writer.write("<!-- ");
        if (comment instanceof String) {
            writer.write((String) comment);
        } else {
            writer.write(comment.toString());
        }
        writer.write(" -->");

    }


    /**
     * <p>Write a newline to the underlying writer, after closing any
     * element that is currently open.</p>
     *
     * @exception IOException if an input/output error occurs
     */
    public void writeNewline() throws IOException {

        close();
        writer.write("\n");

    }


    /**
     * <p>Write an object as body text, after converting it to a String
     * (if necessary), and after performing any escaping appropriate for
     * XML.  If there is an open element that has been created by a call
     * to <code>startElement()</code>, that element will be closed first.</p>
     *
     * @param text Body text to be written
     *
     * @exception IOException if an input/output error occurs
     */
    public void writeText(Object text) throws IOException {

        close();
        if (text instanceof String) {
            string(writer, (String) text);
        } else {
            string(writer, text.toString());
        }
    
    }


    /**
     * <p>Write the end of an element, after closing any open element created
     * by a call to <code>startElement()</code>.  Elements must be closed
     * in the inverse order from which they were opened; it is an error
     * to do otherwise.</p>
     *
     * @param name Element name to be ended
     *
     * @exception IOException if an input/output error occurs
     */
    public void endElement(String name) throws IOException {

        if (open) {
            writer.write("/");
            close();
        } else {
            writer.write("</");
            writer.write(name);
            writer.write(">");
        }
    }


    /**
     * <p>Finish any appropriate epilogue for the ending of this document,
     * and flush any remaining content to the writer.  Afterwards, call
     * <code>flush()</code> on the writer as well.</p>
     *
     * @exception IOException if an input/output error occurs
     */
    public void endDocument() throws IOException {

        close();
        writer.flush();

    }


    // --------------------------------------------------------- Private Methods


    /**
     * <p>Close any element that is currently open.</p>
     *
     * @exception IOException if an input/output error occurs
     */
    private void close() throws IOException {

        if (open) {
            writer.write(">");
            open = false;
        }

    }


    /**
     * <p>Write a numeric character reference for specified character
     * to the specfied writer.</p>
     *
     * @param writer Writer we are writing to
     * @param ch Character to be translated and appended
     */
    private void numeric(Writer writer, char ch) throws IOException {

        writer.write("&#");
        writer.write(String.valueOf(ch));
        writer.write(";");

    }


    /**
     * <p>Write the specified string (after performing suitable
     * replacement of characters by corresponding entities) to the
     * specified writer.</p>
     *
     * @param writer Writer we are writing to
     * @param s String to be filtered and written
     */
    private void string(Writer writer, String s) throws IOException {

        // Process every character in the incoming string
        int n = s.length();
        for (int i = 0; i < n; i++) {

            // Process the next character in the text to be rendered
            char ch = s.charAt(i);
            if (ch <= 0xff) {
                // In single byte characters, replace only the five
                // characters for which well-known entities exist in XML
                if (ch == 0x22) {
                    writer.write("&quot;");
                } else if (ch == 0x26) {
                    writer.write("&amp;");
                } else if (ch == 0x27) {
                    writer.write("&apos;");
                } else if (ch == 0x3C) {
                    writer.write("&lt;");
                } else if (ch == 0X3E) {
                    writer.write("&gt;");
                } else {
                    writer.write(ch);
                }
            } else {
                if (substitution()) {
                    numeric(writer, ch);
                } else {
                    writer.write(ch);
                }
            }

        }

    }


    /**
     * <p>Return true if entity substitution should be performed on double
     * byte character values.</p>
     */
    private boolean substitution() {

        if ("UTF-8".equals(encoding) || "UTF-16".equals(encoding)) {
            return false;
        } else {
            return true;
        }

    }


}
