/*
 * Copyright 2004-2005 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.shale.remote;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Reader;
import java.io.Writer;
import java.util.Locale;
import java.util.Map;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.chain.impl.ContextBase;

/**
 * <p>Implementation of {@link RemoteContext} suitable for use when processing
 * an HTTP request received via the Servlet API.</p>
 *
 * @deprecated Use corresponding functionality from the
 *  <code>org.apache.shale.remoting</code> package
 *
 * $Id: ServletRemoteContext.java 419431 2006-07-06 04:38:39Z wsmoak $
 */
public class ServletRemoteContext extends ContextBase implements RemoteContext {
    

    // -------------------------------------------------------- Static Variables


    /**
     * <p>Buffer size for input and output streams.</p>
     */
    private static final int BYTES_BUFFER_SIZE = 1024;


    /**
     * <p>Buffer size for readers and writers.</p>
     */
    private static final int CHARS_BUFFER_SIZE = 2048;


    // ------------------------------------------------------------- Constructor


    /**
     * <p>Construct a new {@link ServletRemoteContext} relevant for
     * the specified request, response, and context instances.</p>
     *
     * @param context <code>ServletContext</code> for this web application
     * @param request <code>HttpServletRequest</code> for this request
     * @param response <code>HttpServletResponse</code> for this response
     */
    public ServletRemoteContext(ServletContext context,
                                HttpServletRequest request,
                                HttpServletResponse response) {

        this.context = context;
        this.request = request;
        this.response = response;

    }


    // ------------------------------------------------------ Instance Variables


    /**
     * <p>The <code>ServletContext</code> for this web application.</p>
     */
    private ServletContext context = null;


    /**
     * <p>The <code>Map</code> of application scope attributes.</p>
     */
    private Map contextAttributes = null;


    /**
     * <p>The <code>HttpServletRequest</code> for this request.</p>
     */
    private HttpServletRequest request = null;


    /**
     * <p>The <code>Map</code> of request scope attributes.</p>
     */
    private Map requestAttributes = null;


    /**
     * <p>The <code>HttpServletResponse</code> for this response.</p>
     */
    private HttpServletResponse response = null;


    /**
     * <p>The <code>Map</code> of session scope attributes.</p>
     */
    private Map sessionAttributes = null;


    // --------------------------------------------------- RemoteContext Methods


    // Specified by RemoteContext
    public Map getContextAttributes() {

        if (contextAttributes == null) {
            contextAttributes = new ContextAttributes(context);
        }
        return contextAttributes;

    }


    // Specified by RemoteContext
    public Map getRequestAttributes() {

        if (requestAttributes == null) {
            requestAttributes = new RequestAttributes(request);
        }
        return requestAttributes;

    }


    // Specified by RemoteContext
    public int getRequestContentLength() {

        return request.getContentLength();

    }


    // Specified by RemoteContext
    public String getRequestContentType() {

        return request.getContentType();

    }


    // Specified by RemoteContext
    public String getRequestEncoding() {

        return request.getCharacterEncoding();

    }


    // Specified by RemoteContext
    public Locale getRequestLocale() {

        return request.getLocale();

    }


    // Specified by RemoteContext
    public String getRequestMethod() {

        return request.getMethod();

    }


    // Specified by RemoteContext
    public Map getRequestParameters() {

        return request.getParameterMap();

    }


    // Specified by RemoteContext
    public Reader getRequestReader() throws IOException {

        return new BufferedReader(request.getReader(), CHARS_BUFFER_SIZE);

    }

    
    // Specified by RemoteContext
    public InputStream getRequestStream() throws IOException {

        return new BufferedInputStream(request.getInputStream(), BYTES_BUFFER_SIZE);

    }


    private int responseContentLength = 0;


    // Specified by RemoteContext
    public int getResponseContentLength() {

        return responseContentLength;

    }


    // Specified by RemoteContext
    public void setResponseContentLength(int contentLength) {

        this.responseContentLength = contentLength;
        response.setContentLength(contentLength);

    }


    // Specified by RemoteContext
    public String getResponseContentType() {

        return response.getContentType();

    }


    // Specified by RemoteContext
    public void setResponseContentType(String contentType) {

        response.setContentType(contentType);

    }


    // Specified by RemoteContext
    public String getResponseEncoding() {

        return response.getCharacterEncoding();

    }


    // Specified by RemoteContext
    public void setResponseEncoding(String encoding) {

        response.setCharacterEncoding(encoding);

    }


    // Specified by RemoteContext
    public OutputStream getResponseStream() throws IOException {

        return new BufferedOutputStream(response.getOutputStream(), BYTES_BUFFER_SIZE);

    }


    // Specified by RemoteContext
    public Writer getResponseWriter() throws IOException {

        return new BufferedWriter(response.getWriter(), CHARS_BUFFER_SIZE);

    }


    // Specified by RemoteContext
    public Map getSessionAttributes() {

        if (sessionAttributes == null) {
            sessionAttributes = new SessionAttributes(request.getSession());
        }
        return sessionAttributes;

    }


}
